==> express-form-post-1.2.0.tgz: package/lib/attachListeners.js <==
const hasha = require("hasha");
const path = require("path");

module.exports = function(busboy, req) {

	busboy.on("file", (fieldname, file, originalname, encoding, mimetype) => {
		// emit 'end' event on file skip attaching listeners
		if(originalname == "" || req.efp._data[fieldname] != undefined || req.efp._finished) return file.resume();
		req.efp._data[fieldname] = { size: 0 };

		// Busboy emits limit event before my file listeners can reach it
		file.on("limit", () => {
			req.efp.handleError(new Error("File limit reached on file"));
		});

		// validate file
		new Promise((resolve, reject) => {
			// handlePromise is cb in function declaration
			const handlePromise = (validFlag) => {
				if(validFlag == false) {
					reject(new Error("Validation error by custom validateFile function"));
				} else if(validFlag instanceof Error) {
					reject(validFlag);
				} else {
					resolve();
				}
			};
			this.opts.validateFile(fieldname, mimetype, handlePromise);
		})
		.then(() => {
			let fileInfo = {
				originalname: originalname,
				fieldname: fieldname,
				mimetype: mimetype,
				encoding: encoding
			}
			// user may use filename function but incorrectly return nothing. no warning supplied. defaults to hash
			this.opts.filename(fileInfo).then((save_filename) => {
				typeof save_filename == "string" && save_filename.length > 0 ? "" : save_filename = hasha(Date.now() + originalname);
				save_filename.includes("/") ? (
					this.opts.directory = path.join(this.opts.directory, save_filename, ".."),
					save_filename = path.basename(path.resolve(...(save_filename.split("/"))))
				): "";
				let uploadInfo = {
					directory: this.opts.directory,
					filename: save_filename,
					mimetype: mimetype,
					fieldname: fieldname,
					file: file,
					api: this.opts.api
				};

				// init duplex stream (read/writable) or concat-stream depending on store method
				this.storeMethod(uploadInfo, req)
				.then((file_contents) => {
					req.efp._data[fieldname].stream = file_contents;
					// do not end through pipe, since streams should only end through efp finished method
					file.pipe(file_contents, { end: false }); 
					file.on("data", (data) => {
						if(!req.efp._finished) req.efp._data[fieldname].size += data.length;
					});
					file.on("error", (err) => req.efp.handleError(err));
				})
				.catch((err) => {
					req.efp.handleError(err);
				});
			}).catch((err) => {
				req.efp.handleError(err); // this probably will not trigger
			});
		})
		.catch((err) => {
			req.efp.handleError(err);
			return file.resume(); // emit 'end' event on file for busboy 'finish' prevents infinite piping for req
		});
	});

	busboy.on("field", (fieldname, val, fieldnameTruncated, valTruncated) => {
		!valTruncated && !fieldnameTruncated ? req.body[fieldname] = val : "";
	});

	busboy.on("error", (err) => req.efp.handleError(err));
	busboy.on("partsLimit", () => req.efp.handleError("parts limit reached"));
	busboy.on("filesLimit", () => req.efp.handleError("files limit reached"));
	busboy.on("fieldsLimit", () => req.efp.handleError("fields limit reached"));

	
	busboy.on("finish", () => {
		if(req.efp._finished) return; // handleError was called
		req.efp.busboy._finished = true;

		new Promise((resolve, reject) => {
			const handlePromise = (validFlag) => {
				if(validFlag == false) {
					reject(new Error("Validation failed on validateBody function"));
				} else if(validFlag instanceof Error) {
					reject(validFlag);
				} else {
					resolve();
				}
			};
			this.opts.validateBody(req.body, handlePromise);
		})
		.then(() => {
			if(Object.keys(req.efp._data).length == 0) return req.efp.finished();
			// check for min size
			for(let key in req.efp._data) {
				if(this.opts.minfileSize > req.efp._data[key].size) {
					return req.efp.handleError(new Error("Uploaded file was smaller than minfileSize."
						+ " the file was of size " + req.efp._data[key].size));
				}
			}
			// save files after all validation
			for(let key in req.efp._data) {
				req.efp._data[key].stream.end();
			}
		})
		.catch((err) => {
			return req.efp.handleError(err);
		});
	});
};

==> express-form-post-1.2.1.tgz: package/lib/attachListeners.js <==
const hasha = require("hasha");
const path = require("path");

module.exports = function(busboy, req) {

	busboy.on("file", (fieldname, file, originalname, encoding, mimetype) => {
		// emit 'end' event on file skip attaching listeners
		if(originalname == "" || req.efp._data[fieldname] != undefined || req.efp._finished) return file.resume();
		req.efp._data[fieldname] = { size: 0 };

		// Busboy emits limit event before my file listeners can reach it
		file.on("limit", () => {
			req.efp.handleError(new Error("File limit reached on file"));
		});

		// validate file
		new Promise((resolve, reject) => {
			// handlePromise is cb in function declaration
			const handlePromise = (validFlag) => {
				if(validFlag == false) {
					reject(new Error("Validation error by custom validateFile function"));
				} else if(validFlag instanceof Error) {
					reject(validFlag);
				} else {
					resolve();
				}
			};
			this.opts.validateFile(fieldname, mimetype, handlePromise);
		})
		.then(() => {
			let fileInfo = {
				originalname: originalname,
				fieldname: fieldname,
				mimetype: mimetype,
				encoding: encoding
			};
			// user may use filename function but incorrectly return nothing. no warning supplied. defaults to hash
			this.opts.filename(req, fileInfo).then((save_filename) => {
				typeof save_filename == "string" && save_filename.length > 0 ? "" : save_filename = hasha(Date.now() + originalname);
				save_filename.includes("/") ? (
					this.opts.directory = path.join(this.opts.directory, save_filename, ".."),
					save_filename = path.basename(path.resolve(...(save_filename.split("/"))))
				): "";
				let uploadInfo = {
					directory: this.opts.directory,
					filename: save_filename,
					mimetype: mimetype,
					fieldname: fieldname,
					file: file,
					api: this.opts.api
				};

				// init duplex stream (read/writable) or concat-stream depending on store method
				this.storeMethod(uploadInfo, req)
				.then((file_contents) => {
					req.efp._data[fieldname].stream = file_contents;
					// do not end through pipe, since streams should only end through efp finished method
					file.pipe(file_contents, { end: false }); 
					file.on("data", (data) => {
						if(!req.efp._finished) req.efp._data[fieldname].size += data.length;
					});
					file.on("error", (err) => req.efp.handleError(err));
				})
				.catch((err) => {
					req.efp.handleError(err);
				});
			}).catch((err) => {
				req.efp.handleError(err); // this probably will not trigger
			});
		})
		.catch((err) => {
			req.efp.handleError(err);
			return file.resume(); // emit 'end' event on file for busboy 'finish' prevents infinite piping for req
		});
	});

	busboy.on("field", (fieldname, val, fieldnameTruncated, valTruncated) => {
		!valTruncated && !fieldnameTruncated ? req.body[fieldname] = val : "";
	});

	busboy.on("error", (err) => req.efp.handleError(err));
	busboy.on("partsLimit", () => req.efp.handleError("parts limit reached"));
	busboy.on("filesLimit", () => req.efp.handleError("files limit reached"));
	busboy.on("fieldsLimit", () => req.efp.handleError("fields limit reached"));

	
	busboy.on("finish", () => {
		if(req.efp._finished) return; // handleError was called
		req.efp.busboy._finished = true;

		new Promise((resolve, reject) => {
			const handlePromise = (validFlag) => {
				if(validFlag == false) {
					reject(new Error("Validation failed on validateBody function"));
				} else if(validFlag instanceof Error) {
					reject(validFlag);
				} else {
					resolve();
				}
			};
			this.opts.validateBody(req.body, handlePromise);
		})
		.then(() => {
			if(Object.keys(req.efp._data).length == 0) return req.efp.finished();
			// check for min size
			for(let key in req.efp._data) {
				if(this.opts.minfileSize > req.efp._data[key].size) {
					return req.efp.handleError(new Error("Uploaded file was smaller than minfileSize."
						+ " the file was of size " + req.efp._data[key].size));
				}
			}
			// save files after all validation
			for(let key in req.efp._data) {
				req.efp._data[key].stream.end();
			}
		})
		.catch((err) => {
			return req.efp.handleError(err);
		});
	});
};
